/*
 * Copyright (c) 2014 LAAS/CNRS
 * Copyright (c) 2008-2014 Max Planck Institute for Biological Cybernetics
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 */
#include "acheader.h"

#include "data_struct.h"
#include "revision.h"
#include "main.h"
#include "nhfc.h"

struct MikKopData mikKopData = {
		0,// signed int	rawAccX; // 0 // from 0 to 1024
		0,// signed int	rawAccY; // 1 // from 0 to 1024
		0,// signed int	rawAccZ; // 2 // from 0 to 1024
		0,// signed int	rawGyroX; // 3 // from 0 to 1024
		0,// signed int	rawGyroY; // 4 // from 0 to 1024
		0,// signed int	rawGyroZ; // 5 // from 0 to 1024
		0,// signed int	accX; // 6 // in g
		0,// signed int	accY; // 7 // in g
		0,// signed int	accZ; // 8 // in g
		0,// signed int	rollRate; // 9  // deg/s
		0,// signed int	pitchRate; // 10 // deg/s
		0,// signed int	yawRate; // 11 // deg/s

		0,// unsigned int	battVoltage; // 12

		//sensor parameters
		512,// signed int offsetRawAccX;  // 13 // from 0 to 1024
		512,// signed int offsetRawAccY;  // 14 // from 0 to 1024
		512,// signed int	offsetRawAccZ;  // 15 // from 0 to 1024
		0,// signed int	driftGyroX; // 16 //
		0,// signed int	driftGyroY; // 17 //
		0,// signed int	driftGyroZ; // 18 //



		//...

		//estimated data
		0,// signed int	pitch; // 19 // //deg
		0,// signed int	roll;  // 20 // //deg
		0,// signed int	pitchRateLowPass; // 21 //
		0,// signed int rollRateLowPass;  // 22 //

		//control parameters
		0,// unsigned int propGain;  // 23 //
		0,// unsigned int derivGain; // 24 //
		0,// unsigned int integGain; // 25 //
		0,// unsigned int yawRateLowPassGain; // 26 //
		0,// unsigned int yawAccGain;  // 27 //
		//..

		//logic parameters
		0,// unsigned int yawControlType; // 28 // /// 0 :  yaw rate from gyroscope, 1 :  yaw rate from vicon
		5,// unsigned int dataTimePeriod; // 29 //

		//time
		0,// unsigned int time; // 30 //

		//bool to see if Data is send
		0, // unsigned int mirrorDataActive; // 31 // <- Do not send Mirror Data by default

		MOTOR_STATE_OFF, // signed int motorState; // 32 //
		0,// unsigned int driftEstimActive;// 33 // <- will be set to 1 during the estimation and to 0 before the flight.

		0,// unsigned int aveReceiveCmdPeriodMs; // 34 // This does not to get set in the control loop.
		0,// unsigned int maxReceiveCmdPeriodMs; // 35 // This does not to get set in the control loop.

		0,// unsigned int aveSendPeriodMs; // 36 // This does not to get set in the control loop.
		0,// unsigned int maxSendPeriodMs; // 37 // This does not to get set in the control loop.

		0,// unsigned int aveFCPeriodMs; // 38 // This does not to get set in the control loop.
		0,// unsigned int maxFCPeriodMs; // 39 // This does not to get set in the control loop.

		2,//unsigned int robotId; // 40 // This not get set in the control loop
		SVN_REVISION,//unsigned int firmwareRevision; // 41 // This not get set in the control loop

		NEAR_HOV_ONBOARD,//unsigned int flightCtrlMode; // 42 //

		0,// signed int rawAtmoPress; // 43 // likely from 0 to 1024


		0,// signed int	yawRateLowPass; // 44 // //deg

		0,// unsigned int cntCRCErrors; // 45

		MIN_PER_MS_DEC,// minFlightCtrlPerMsDecs; // 46 //
		MIN_PER_MS_DEC,// minEstPerMsDecs; // 47 //
		MIN_PER_MS_DEC,// minExtCmdProcessPerMsDecs; // 48 //
		MIN_PER_MS_DEC,// minSendDataPerMsDecs; // 49 //
		MIN_PER_MS_DEC,// minImuTransPerMsDecs; // 50 //

		0,// stateEstStepsPerSec; // 51 //

		1, // secondarySerialEnabled; // 52 //
		0, // sendPacketsPerSec1; // 53 //
		80, // emergencyThrust; // 54 //
		400, // emergencyTimeout; //55 //
		0, // PortPinJ3; //56 //
		0, // desThrust; //57 //
		0, // rollTorque; //58 //
		0, // pitchTorque; //59 //
		0, // yawTorque; //60 //
		0, // motorBuf0; //61 //
		0, // motorBuf1; //62 //
		0, // motorBuf2; //63 //
		0, // motorBuf3; //64 //
		100, // satValue; // 65 //
		25 // yawsatValue; // 66 //
};



// set parameters (id) with value. Mapping is inverse of updateMikKopData
void setValueByID(unsigned char id, signed int value) {

	switch(id) {
	case 0: // mikKopData.rawAccX // from 0 to 1024 read-only
		break;

	case 1: // mikKopData.rawAccY // from 0 to 1024 read-only
		break;

	case 2: // mikKopData.rawAccZ // from 0 to 1024 read-only
		break;

	case 3: // mikKopData.rawGyroX // from 0 to 1024 read-only
		break;

	case 4: // mikKopData.rawGyroY // from 0 to 1024 read-only
		break;

	case 5:  // mikKopData.rawGyroZ = gier2acc; // from 0 to 1024
		//gier2acc = value;
		break;

	case 6: // 	mikKopData.accX // in g read-only
		// todo
		break;

	case 7: // 	mikKopData.accY // in g read-only
		// todo
		break;

	case 8: // 	mikKopData.accZ // in g read-only
		// todo
		break;

	case 9:  // mikKopData.rollRate // dec of deg/sec read-only
		break;

	case 10: // mikKopData.pitchRate // dec of deg/sec read-only
		break;

	case 11: // mikKopData.yawRate // dec of deg/sec read-only
		break;

	case 12: // mikKopData.	battVoltage // read-only
		break;

	case 13: // mikKopData.offsetRawAccX // from 0 to 1024
		mikKopData.offsetRawAccX = value;
		break;

	case 14: // mikKopData.offsetRawAccY // from 0 to 1024
		mikKopData.offsetRawAccY = value;
		break;

	case 15: // mikKopData.offsetRawAccZ // from 0 to 1024
		mikKopData.offsetRawAccZ = value;
		break;

	case 16: // mikKopData.driftGyroX // thousands of deg/sec
		mikKopData.driftGyroX = value;
		break;

	case 17: // mikKopData.driftGyroY // thousands of deg/sec
		mikKopData.driftGyroY = value;
		break;

	case 18: // mikKopData.driftGyroZ // thousands of deg/sec
		mikKopData.driftGyroZ = value;
		break;

	case 19: // mikKopData.pitch // dec of deg read-only
		break;

	case 20: //	mikKopData.roll // dec of deg read-only
		break;

	case 21: // mikKopData.pitchRateLowPass // dec of deg/sec read-only
		break;

	case 22: // mikKopData.rollRateLowPass // dec of deg/sec read-only
		break;

	case 23: // mikKopData.propGain
		//kp_att = (signed long int)value * KP_ATT_MULT;
		mikKopData.propGain = value;
		break;

	case 24: // mikKopData.derivGain
		//kd_att = (signed long int)value * KD_ATT_MULT;
		mikKopData.derivGain = value;
		break;

	case 25: // mikKopData.integGain
		//ki_att = (signed long int)value * KI_ATT_MULT;
		mikKopData.integGain = value;
		break;

	case 26: // mikKopData.yawRateGain
		//kp_yaw = (signed long int)value * KP_YAW_MULT;
		mikKopData.yawRateGain = value;
		break;

	case 27: // mikKopData.yawAccGain
		//kp_yaw_acc = (signed long int)value * KP_YAW_ACC_MULT;
		mikKopData.yawAccGain = value;
		break;

	case 28: // mikKopData.yawControlType // 1:  yaw rate from gyroscope, 2:  yaw rate from vicon
		mikKopData.yawControlType = (unsigned char) value;
		break;

	case 29: // mikKopData.dataTimePeriod // Mirroring Time Step per 10ms
		mikKopData.dataTimePeriod = (unsigned char) value;
		break;

	case 30: // mikKopData.timeMsMod60s //in ms (mod 60 000ms)
		break;

	case 31: // mikKopData.mirrorDataActive // 0: mirror not active, 1: mirror active
		mikKopData.mirrorDataActive = value;
		break;

	case 32: // mikKopData.motorState // 0: init, 1: on, 2: emergency, 3: off

		if (value == MOTOR_STATE_EMER){
			RED_LED_ON;
		} else {
			RED_LED_OFF;
		}

		if (mikKopData.motorState != value && mikKopData.motorState == 1 && mikKopData.flightCtrlMode == 1) {
			mikKopData.minEstPerMsDecs = 20;
			nearHovEstUpdate(1); // Reset values when going back to nearHoverOnboard
		}
		if (value == 0) {
			mikKopData.minEstPerMsDecs = 20;
			nearHovEstUpdate(1); // Reset values when motorState is set to init
		}
		mikKopData.motorState = value;
		break;

	case 33: // mikKopData.driftEstimActive // 0: gyro drift estimation not active, 1: gyro drift estimation active
		mikKopData.driftEstimActive = value;
		break;

	case 34: // mikKopData.receiveCmdsPerSec // read-only
		break;

	case 35: // mikKopData.maxReceiveCmdPeriodMs // read-only
		break;

	case 36: // mikKopData.sendPacketsPerSec // read-only
		break;

	case 37: // mikKopData.maxSendPeriodMs // read-only
		break;

	case 38: // mikKopData.ctrlStepsPerSec // read-only
		break;

	case 39: // mikKopData.maxFCPeriodMs // read-only
		break;

	case 40: // mikKopData.robotId // read-only
		break;

	case 41: // mikKopData.firmwareRevision // read-only
		break;

	case 42: // mikKopData.flightCtrlMode // 0: near-hovering ctrl, 1: directly apply commands
		if (value == 0 && mikKopData.flightCtrlMode == 1) {
			mikKopData.minEstPerMsDecs = 20;
			nearHovEstUpdate(1); // Reset values when going back to nearHoverOnboard
		}
		mikKopData.flightCtrlMode = value;
		break;

	case 43: // mikKopData.rawAtmoPress  // read-only // from 0 to 1024
		break;

	case 44: // yawRateLowPass // dec of deg/sec read-only
		break;

	case 45: // cntCRCErrors
		mikKopData.cntCRCErrors = value;
		break;

	case 46: // minFlightCtrlPerMsDecs
		mikKopData.minFlightCtrlPerMsDecs = value;
		break;

	case 47: // minEstPerMsDecs
		// Check for allowed range (20-35) if not external controlled
		if (mikKopData.motorState == MOTOR_STATE_ON && mikKopData.flightCtrlMode == SMURF_CTRL_MODE) {
			mikKopData.minEstPerMsDecs = value;
		} else {
			if (value > 35)
				mikKopData.minEstPerMsDecs = 35;
			else if (value < 20)
				mikKopData.minEstPerMsDecs = 20;
			else
				mikKopData.minEstPerMsDecs = value;
		}
		break;

	case 48: // minExtCmdProcessPerMsDecs
		mikKopData.minExtCmdProcessPerMsDecs = value;
		break;

	case 49: // minSendDataPerMsDecs
		mikKopData.minSendDataPerMsDecs = value;
		break;

	case 50: // minImuTransPerMsDecs
		mikKopData.minImuTransPerMsDecs = value;
		break;

	case 51: // stateEstStepsPerSec
		break;

	case 52: // secondarySerialEnabled
		mikKopData.secondarySerialEnabled = value;
		break;

	case 53: // sendPacketsPerSec1 // read-only
		break;

	case 54: // emergencyThrust
		mikKopData.emergencyThrust = value;
		break;

	case 55: // emergencyTimeout
		mikKopData.emergencyTimeout = value;
		break;

	case 56: // PortPinJ3
	 break;

	case 57: // desThrust
	 break;

	case 58: // rollTorque
	 break;

	case 59: // pitchTorque
	 break;

	case 60: // yawTorque
	 break;

	case 61: // motorBuf0
		break;

	case 62: // motorBuf1
		break;

	case 63: // motorBuf2
		break;

	case 64: // motorBuf3
		break;

// 	case 65: // satValue
// 	mikKopData.satValue = value;
// 	break;
//
// 	case 66: // yawsatValue
// 	mikKopData.yawsatValue = value;
// 	break;

	default:
		break;
	}

}

// get parameters (id) with value.
const signed int getValueByID(unsigned char id) {
	switch (id){
	case 0:
		return mikKopData.rawAccX;
	case 1:
		return mikKopData.rawAccY;
	case 2:
		return mikKopData.rawAccZ;
	case 3:
		return mikKopData.rawGyroX;
	case 4:
		return mikKopData.rawGyroY;
	case 5:
		return mikKopData.rawGyroZ;
	case 6:
		return mikKopData.accX;
	case 7:
		return mikKopData.accY;
	case 8:
		return mikKopData.accZ;
	case 9:
		return mikKopData.rollRate;
	case 10:
		return mikKopData.pitchRate;
	case 11:
		return mikKopData.yawRate;
	case 12:
		return mikKopData.battVoltage;
		//...TODO magnetometer, barometer, motor speeds?

		//sensor parameters
	case 13:
		return mikKopData.offsetRawAccX;
	case 14:
		return mikKopData.offsetRawAccY;
	case 15:
		return mikKopData.offsetRawAccZ;
	case 16:
		return mikKopData.driftGyroX;
	case 17:
		return mikKopData.driftGyroY;
	case 18:
		return mikKopData.driftGyroZ;
	case 19:
		return mikKopData.pitch;
	case 20:
		return mikKopData.roll;
	case 21:
		return mikKopData.pitchRateLowPass;
	case 22:
		return mikKopData.rollRateLowPass;

		//control parameters
	case 23:
		return mikKopData.propGain;
	case 24:
		return mikKopData.derivGain;
	case 25:
		return mikKopData.integGain;
	case 26:
		return mikKopData.yawRateGain;
	case 27:
		return mikKopData.yawAccGain;

		//logic parameters
	case 28:
		return mikKopData.yawControlType;
	case 29:
		return mikKopData.dataTimePeriod;

		//time
	case 30:
		return mikKopData.timeMsMod30s;

		// bool to see if Data is send.
	case 31:
		return mikKopData.mirrorDataActive;

		// enum which integrates commandsActive.
	case 32:
		return mikKopData.motorState;

		// bool to see if drift estimation must be done
	case 33:
		return mikKopData.driftEstimActive;

		// Statistic VARIABLES
	case 34:
		return mikKopData.receiveCmdsPerSec;
	case 35:
		return mikKopData.maxReceiveCmdPeriodMs;

	case 36:
		return mikKopData.sendPacketsPerSec;
	case 37:
		return mikKopData.maxSendPeriodMs;
	case 38:
		return mikKopData.ctrlStepsPerSec;
	case 39:
		return mikKopData.maxFCPeriodMs;
	case 40:
		return mikKopData.robotId;
	case 41:
		return mikKopData.firmwareRevision;
	case 42:
		return mikKopData.flightCtrlMode;
	case 43:
		return mikKopData.rawAtmoPress;
	case 44:
		return mikKopData.yawRateLowPass;
	case 45:
		return mikKopData.cntCRCErrors;
	case 46:
		return mikKopData.minFlightCtrlPerMsDecs;
	case 47:
		return mikKopData.minEstPerMsDecs;
	case 48:
		return mikKopData.minExtCmdProcessPerMsDecs;
	case 49:
		return mikKopData.minSendDataPerMsDecs;
	case 50:
		return mikKopData.minImuTransPerMsDecs;
	case 51:
		return mikKopData.stateEstStepsPerSec;
	case 52:
		return mikKopData.secondarySerialEnabled;
	case 53:
		return mikKopData.sendPacketsPerSec1;
	case 54:
		return mikKopData.emergencyThrust;
	case 55:
		return mikKopData.emergencyTimeout;

	case 56: // PortPinJ3
		return mikKopData.PortPinJ3;


	case 57: // desThrust
		return mikKopData.desThrust;


	case 58: // rollTorque
		return mikKopData.rollTorque;


	case 59: // pitchTorque
		return mikKopData.pitchTorque;


	case 60: // yawTorque
		return mikKopData.yawTorque;


	case 61: // motorBuf0
		return mikKopData.motorBuf0;


	case 62: // motorBuf1
		return mikKopData.motorBuf1;


	case 63: // motorBuf2
		return mikKopData.motorBuf2;

	case 64: // motorBuf3
		return mikKopData.motorBuf3;

// 	case 65: // satValue
// 		return mikKopData.satValue;
//
// 	case 66: // yawsatValue
// 		return mikKopData.yawsatValue;

	default:
		return -1;
	}

}



// Only for Debug messages
/*const char* mikKopDataNames(unsigned int id){
	switch (id){
	case 0:
		return "rawAccX";
	case 1:
		return "rawAccY";
	case 2:
		return "rawAccZ";
	case 3:
		return "rawGyroX";
	case 4:
		return "rawGyroY";
	case 5:
		return "rawGyroZ";
	case 6:
		return "accX";
	case 7:
		return "accY";
	case 8:
		return "accZ";
	case 9:
		return "rollRate";
	case 10:
		return "pitchRate";
	case 11:
		return "yawRate";
	case 12:
		return "battVoltage";
	case 13:
		return "offsetRawAccX";
	case 14:
		return "offsetRawAccY";
	case 15:
		return "offsetRawAccZ";
	case 16:
		return "driftGyroX";
	case 17:
		return "driftGyroY";
	case 18:
		return "driftGyroZ";
	case 19:
		return "pitch";
	case 20:
		return "roll";
	case 21:
		return "pitchRateLowPass";
	case 22:
		return "rollRateLowPass";
	case 23:
		return "propGain";
	case 24:
		return "derivGain";
	case 25:
		return "integGain";
	case 26:
		return "yawRateGain";
	case 27:
		return "yawAccGain";
	case 28:
		return "yawControlType";
	case 29:
		return "dataTimePeriod";
	case 30:
		return "timeMsMod60s";
	case 31:
		return "mirrorDataActive";
	case 32:
		return "motorState";
	case 33:
		return "driftEstimActive";
	case 34:
		return "receiveCmdsPerSec";
	case 35:
		return "maxReceiveCmdPeriodMs";
	case 36:
		return "sendPacketsPerSec";
	case 37:
		return "maxSendPeriodMs";
	case 38:
		return "ctrlStepsPerSec";
	case 39:
		return "maxFCPeriodMs";
	case 40:
		return "robotId";
	case 41:
		return "firmwareRevision";
	case 42:
		return "flightCtrlMode";
	case 43:
		return "rawAtmoPress";
	case 44:
		return "yawRateLowPass";
	case 45:
		return "cntCRCErrors";
	case 46:
		return "minFlightCtrlPerMsDecs";
	case 47:
		return "minEstPerMsDecs";
	case 48:
		return "minExtCmdProcessPerMsDecs";
	case 49:
		return "minSendDataPerMsDecs";
	case 50:
		return "minImuTransPerMsDecs";
	case 51:
		return "stateEstStepsPerSec";
	case 52:
		return "secondarySerialEnabled";
	case 53:
		return "sendPacketsPerSec1";
	case 54:
		return "emergencyThrust";
	default:
		return "unknown";
	}
}*/
