/*
 * Copyright (c) 2014 LAAS/CNRS
 * Copyright (c) 2008-2014 Max Planck Institute for Biological Cybernetics
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 */
#ifndef _DATA_STRUCT_H
#define _DATA_STRUCT_H

#define MIN_PER_MS_DEC 15 // in decs of ms

void setValueByID(unsigned char id, signed int value);
const signed int getValueByID(unsigned char id);


enum MotorState{
	MOTOR_STATE_INIT=0,
	MOTOR_STATE_ON=1,
	MOTOR_STATE_EMER=2,
	MOTOR_STATE_OFF=3
};
typedef enum MotorState MotorState;


enum CtrlMode{
	NEAR_HOV_ONBOARD=0,
	SMURF_CTRL_MODE=1
};
typedef enum CtrlMode CtrlMode;


enum YawCtrlMode{
	YAW_RATE=1, /// 1 :  yaw rate from gyroscope, 2 :  yaw rate from vicon
	YAW_TORQUE=2
};
typedef enum YawCtrlMode YawCtrlMode;


struct MikKopData{
	/*sensor data*/
	signed int rawAccX; // 0 // from 0 to 1024
	signed int rawAccY; // 1 // from 0 to 1024
	signed int rawAccZ; // 2 // from 0 to 1024
	signed int rawGyroX; // 3 // from 0 to 1024
	signed int rawGyroY; // 4 // from 0 to 1024
	signed int rawGyroZ; // 5 // from 0 to 1024
	signed int accX; // 6 // in g
	signed int accY; // 7 // in g
	signed int accZ; // 8 // in g
	signed int rollRate; // 9  // deg/s
	signed int pitchRate; // 10 // deg/s
	signed int yawRate; // 11 // deg/s

	unsigned int battVoltage; // 12

	/*sensor parameters*/
	signed int offsetRawAccX;  // 13 // from 0 to 1024
	signed int offsetRawAccY;  // 14 // from 0 to 1024
	signed int offsetRawAccZ;  // 15 // from 0 to 1024
	signed int driftGyroX; // 16 //
	signed int driftGyroY; // 17 //
	signed int driftGyroZ; // 18 //

	/*estimated data*/
	signed int pitch; // 19 // //deg
	signed int roll;  // 20 // //deg
	signed int pitchRateLowPass; // 21 // //deg/s
	signed int rollRateLowPass;  // 22 // //deg/s

	/*control parameters*/
	unsigned int propGain;  // 23 //
	unsigned int derivGain; // 24 //
	unsigned int integGain; // 25 //
	unsigned int yawRateGain; // 26 //
	unsigned int yawAccGain;  // 27 //

	// enum which indicates how to control the yaw
	unsigned int yawControlType; // 28 //

	unsigned int dataTimePeriod; // 29 //

	/*time in ms modulo 30s*/
	unsigned int timeMsMod30s; // 30 //

	/* bool to see if Data is send.*/
	unsigned int mirrorDataActive; // 31 //

	/* enum which indicates motor status. */
	unsigned int motorState; // 32 //

	/* bool to see if drift estimation must be done */
	unsigned int driftEstimActive;// 33 //

	/* Statistic VARIABLES*/
	unsigned int receiveCmdsPerSec; // 34 //
	unsigned int maxReceiveCmdPeriodMs; // 35 //

	unsigned int sendPacketsPerSec; // 36 //
	unsigned int maxSendPeriodMs; // 37 //

	unsigned int ctrlStepsPerSec; // 38 //
	unsigned int maxFCPeriodMs; // 39 //

	unsigned int robotId; // 40 //
	unsigned int firmwareRevision; // 41 //

	// enum which indicates how to interpret the commands
	unsigned int flightCtrlMode; // 42 //

	signed int rawAtmoPress; // 43 //

	signed int yawRateLowPass;  // 44 // //deg/s

	unsigned int cntCRCErrors; // 45 //

	unsigned int minFlightCtrlPerMsDecs; // 46 //
	unsigned int minEstPerMsDecs; // 47 //
	unsigned int minExtCmdProcessPerMsDecs; // 48 //
	unsigned int minSendDataPerMsDecs; // 49 //
	unsigned int minImuTransPerMsDecs; // 50 //

	unsigned int stateEstStepsPerSec; // 51 //

	unsigned int secondarySerialEnabled; // 52 //

	unsigned int sendPacketsPerSec1; // 53 //

	unsigned int emergencyThrust; // 54 //

	unsigned int emergencyTimeout; // 55 //

	unsigned int PortPinJ3; // 56 //

	signed int desThrust; // 57 //
	signed int rollTorque; // 58 //
	signed int pitchTorque; // 59 //
	signed int yawTorque; // 60 //

	signed int motorBuf0; // 61 //
	signed int motorBuf1; // 62 //
	signed int motorBuf2; // 63 //
	signed int motorBuf3; // 64 //

	signed int satValue; // 65 //
	signed int yawsatValue; // 66 //
};

extern struct MikKopData mikKopData;


//Struct names for debug messagges
//const char* mikKopDataNames(unsigned int index);



#endif //_DATA_STRUCT_H
