/*
 * Copyright (c) 2014-2015 LAAS/CNRS
 * Copyright (c) 2008-2014 Max Planck Institute for Biological Cybernetics
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 */
#include <stdio.h>

#include "data_struct.h"
#include "main.h"
#include "nhfc.h"
#include "uart.h"

volatile unsigned int receivedCmds = 0, ctrlSteps = 0, sendPackets = 0, sendPackets1 = 0, stateEstSteps = 0;

void ctrlStep(void);
unsigned int nextClockResetStats = 0;

void SendMotorData(void);

/* board version */
unsigned char fc_version;

/* stdout */
FILE uart_output = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);


/* --- main ---------------------------------------------------------------- */

int
main(void)
{
  /* detect board version */
  DDRB = 0; PORTB = 0;		/* green led, no pullup */
  DDRD = 0; PORTD = 0x10;	/* J4, pullup */

  if (PINB & 0x2) /* green led on VCC */ {
    if (PIND & 0x10) /* J4 not on GND */
      fc_version = 21;
    else
      fc_version = 22;
  } else
    fc_version = 25;

  /* configure gpio */
	DDRC  = 0x81; // SCL
	DDRC  |=0x40; // HEF4017 Reset
	PORTC = 0xff; // Pull-up SDA
	DDRB  = 0x1B; // LEDs and Offset
	PORTB = 0x01; // LED_Rot
	DDRD  = 0x3a; // Speaker & TXD & J3 J4 J5
	PORTD = 0x4f; // LE
	HEF4017R_ON;
	MCUSR &=~(1<<WDRF);
	WDTCSR |= (1<<WDCE)|(1<<WDE);
	WDTCSR = 0;
	GRN_LED_ON;
	RED_LED_OFF;

	Timer_Init();
	UART_Init();
        stdout = &uart_output;

        tk3_load_settings();

	ADC_Init();
	TWI_Master_Initialise();

	sei(); //enable interrupts

	calibrateAtmoPress();

	while (1)
	{

		// Do a control step
		if (flightCtrlPerMsDecs >= mikKopData.minFlightCtrlPerMsDecs){
			ctrlStep();
		}

		// Do state estimation
		if (estPerMsDecs >= mikKopData.minEstPerMsDecs){
			nearHovEstUpdate(0);
		}

		// If SMURF is being used, send imu data on the secondary serial
		if (imuTransPerMsDecs >= mikKopData.minImuTransPerMsDecs
				&& mikKopData.secondarySerialEnabled){
			IMUTransmission();
		}

		// Process command from mikrokopter interface
		if (extCmdProcessPerMsDecs >= mikKopData.minExtCmdProcessPerMsDecs){
			processRxData();
		}

		// Send data to mikrokopter interface
		if (sendDataPerMsDecs >= mikKopData.minSendDataPerMsDecs){
			sendTxData0();
		}

	}

	return (1);
}





void ctrlStep(void){

	// Update statistics
	flightCtrlPerMsDecs = 0;
	ctrlSteps++;

	if (CountMillisecondsLastCtrlStep>mikKopData.maxFCPeriodMs){
		mikKopData.maxFCPeriodMs = CountMillisecondsLastCtrlStep;
	}
	CountMillisecondsLastCtrlStep = 0;

	// Compute motor commands depending on the state
	switch(mikKopData.motorState)
	{
	case MOTOR_STATE_INIT:{
		nearHovCmds.pitch = 0;
		nearHovCmds.roll = 0;
		nearHovCmds.thrust = 32;
		nearHovCmds.yawCmd = 0;
		nearHovCtrlStep();
		break;
	}

	case MOTOR_STATE_ON:

		if (CountMillisecondsLastRx0>mikKopData.emergencyTimeout){
			//printf("Command timeout! Going to emergency\r\n");
			mikKopData.motorState = MOTOR_STATE_EMER;
			mikKopData.minEstPerMsDecs = 20;
			nearHovEstUpdate(1);  // reset values when going back to nearHoverOnboard
			RED_LED_ON;
		} else if (mikKopData.flightCtrlMode == NEAR_HOV_ONBOARD){
			memcpy((unsigned char *)&nearHovCmds, (unsigned char *)&cmdBuf, sizeof(nearHovCmds));
			nearHovCtrlStep();
		} else {
			memcpy((unsigned char *)motorBuf, (unsigned char *)&cmdBuf, sizeof(cmdBuf));
		}
		break;

	case MOTOR_STATE_EMER:
		nearHovCmds.pitch=0;
		nearHovCmds.roll=0;
		nearHovCmds.thrust = mikKopData.emergencyThrust;
		nearHovCmds.yawCmd = 0;
		nearHovCtrlStep();

		if (CountMillisecondsInEmergency > 1800)
		{
			mikKopData.motorState = MOTOR_STATE_OFF;
			CountMillisecondsInEmergency = 0;
			RED_LED_OFF;
		}

		break;

	case MOTOR_STATE_OFF:
		motorBuf[0] = 0;
		motorBuf[1] = 0;
		motorBuf[2] = 0;
		motorBuf[3] = 0;
		break;

	default:
		motorBuf[0] = 0;
		motorBuf[1] = 0;
		motorBuf[2] = 0;
		motorBuf[3] = 0;
		mikKopData.motorState = MOTOR_STATE_OFF;
		break;

	}

	// Send commands to the motors
	SendMotorData();
}
