/*
 * Copyright (c) 2014 LAAS/CNRS
 * Copyright (c) 2008-2014 Max Planck Institute for Biological Cybernetics
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 */
#include "acheader.h"

#include "main.h"
#include "data_struct.h"


volatile unsigned int CountClockOverflow = 0;
volatile unsigned int CountMillisecondsLastTx0 = 0, CountMillisecondsInEmergency = 0;
//		CountMillisecondsLastTx1 = 0,

volatile unsigned int CountMillisecondsLastRx0 = 0;
volatile unsigned int CountMillisecondsLastCtrlStep = 0;

volatile unsigned int CountCalibAtmoPress1kHz = 0;

// Rate limiters
volatile unsigned int flightCtrlPerMsDecs = MIN_PER_MS_DEC;
volatile unsigned int estPerMsDecs = MIN_PER_MS_DEC;
volatile unsigned int extCmdProcessPerMsDecs = MIN_PER_MS_DEC;
volatile unsigned int sendDataPerMsDecs = MIN_PER_MS_DEC;
volatile unsigned int imuTransPerMsDecs = MIN_PER_MS_DEC;

volatile unsigned char mirrorTimer = 0;

enum {
  STOP             = 0,
  CK               = 1,
  CK8              = 2,
  CK64             = 3,
  CK256            = 4,
  CK1024           = 5,
  T0_FALLING_EDGE  = 6,
  T0_RISING_EDGE   = 7
};


#if defined (__AVR_ATmega1284P__)
SIGNAL(TIMER0_OVF_vect)
#else
SIGNAL(SIG_OVERFLOW0)    // 9765kHz
#endif
{

	static unsigned char cnt = 0;
	static unsigned int cnt2 = 0;

	CountClockOverflow++;

	// Rate limiters
	flightCtrlPerMsDecs++;
	estPerMsDecs++;
	extCmdProcessPerMsDecs++;
	sendDataPerMsDecs++;
	imuTransPerMsDecs++;

	// The following runs at 1 KHz
	if(!cnt--)
    {
		cnt = 8;

		CountCalibAtmoPress1kHz++;

		// TODO: This will be casted into a signed, then the actual limit is 32767
		mikKopData.timeMsMod30s = (unsigned int) (((CountClockOverflow*10)/97) % 30000);

		if (mikKopData.motorState == MOTOR_STATE_EMER){
			CountMillisecondsInEmergency++;
		}

		CountMillisecondsLastRx0++;
		if (mikKopData.mirrorDataActive){
			CountMillisecondsLastTx0++;
			mirrorTimer++;
		}

/*		if (mikKopData.secondarySerialEnabled){
			CountMillisecondsLastTx1++;
		}*/
    }


	// The following runs at 1 Hz
	if(!cnt2--)
	{
		cnt2 = 9764;

		mikKopData.receiveCmdsPerSec = receivedCmds;
		receivedCmds = 0;

		mikKopData.ctrlStepsPerSec = ctrlSteps;
		ctrlSteps = 0;

		mikKopData.sendPacketsPerSec = sendPackets;
		sendPackets = 0;

		mikKopData.stateEstStepsPerSec = stateEstSteps;
		stateEstSteps = 0;

		mikKopData.sendPacketsPerSec1 = sendPackets1;
		sendPackets1 = 0;
	}

}


//----------------------------
void Timer_Init(void)
{
    TCCR0B = CK8;
    TCCR0A = (1<<COM0A1)|(1<<COM0B1)|3; // fast PWM mode
    OCR0A =  0;
    OCR0B = 120;
    TCNT0 = (unsigned char)-TIMER_RELOAD_VALUE;  // reload

    TIMSK0 |= _BV(TOIE0);
}
