/*
 * Copyright (c) 2014 LAAS/CNRS
 * Copyright (c) 2008-2014 Max Planck Institute for Biological Cybernetics
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 */
#include "acheader.h"

#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <avr/pgmspace.h>

#include "uart.h"
#include "main.h"
#include "data_struct.h"
#include "timer0.h"

#define FC_ADDRESS 1
#define NC_ADDRESS 2
#define MK3MAG_ADDRESS 3

#define ABO_TIMEOUT 4000 // disable abo after 4 seconds
#define MAX_TX_BUFF  160
#define MAX_RX_BUFF  160

#ifdef BAUD_RATE_FACTOR
static const unsigned int ubrr = BAUD_RATE_FACTOR;
#else
static const unsigned int ubrr = 21; // default 115200
#endif

unsigned volatile char TransmissionComplete0 = 1;
unsigned volatile char TransmissionComplete1 = 1;

unsigned volatile char TxdBuffer0[MAX_TX_BUFF];
unsigned volatile char RxdBuffer0[MAX_RX_BUFF];
unsigned volatile char TxdBuffer1[MAX_TX_BUFF];
unsigned volatile char RxdBuffer1[MAX_RX_BUFF];


unsigned volatile char newDataReceived0 = 0;
unsigned volatile char numReceivedBytes0 = 0;

struct str_CommandsPacket cmdBuf={0,0,0,0};

struct str_SingleValue singleUpdateValue;
struct str_SingleValue singleSetValue;
struct str_CommandsPlusValuePacket commandsPlusValue;
struct str_CommandsPlusIdPacket commandsPlusId;
struct str_CommandsPlusDataIdsPacket commandsPlusDataIds;

static const unsigned int mikKopDataSize = sizeof(struct MikKopData)/sizeof(signed int);

#define MIK_KOP_DATA_PACKET_SIZE 10



unsigned char confirmActiveDataIds=0,confirmSingleUpdateValue=0,confirmSingleSetValue=0;


unsigned char activeDataIds[MIK_KOP_DATA_PACKET_SIZE] = {
	0,1,2,3,4,5,6,7,8,9
};



unsigned char pRxData = 0;
unsigned char RxDataLen = 0;

const unsigned int computeCRC(volatile unsigned char *checksumData, int bytes){

	unsigned int crc = 0;
        int j;
	for (j = 0; j < bytes; j++) {
		crc += checksumData[j];
	}

	return (crc & 0xFFF); //only lower 12 bits
}

const unsigned char checkCRC(volatile unsigned char *checksumData, int bytes){

	const unsigned int crcIn = ((checksumData[bytes-2]-'=') << 6)
								| (checksumData[bytes-1] - '=');
	return (crcIn == computeCRC(checksumData,bytes-2));
}


unsigned char* decode64(volatile unsigned char *buffer, unsigned int nbytes)
{
	unsigned char a,b,c,d;
	unsigned char x,y,z;
	unsigned char ptrIn = 3;
	unsigned char ptrOut = 3;
	unsigned char len = nbytes - 6;

	while(len)
	{
		a = buffer[ptrIn++] - '=';
		b = buffer[ptrIn++] - '=';
		c = buffer[ptrIn++] - '=';
		d = buffer[ptrIn++] - '=';

		x = (a << 2) | (b >> 4);
		y = ((b & 0x0f) << 4) | (c >> 2);
		z = ((c & 0x03) << 6) | d;

		if(len--) buffer[ptrOut++] = x; else break;
		if(len--) buffer[ptrOut++] = y; else break;
		if(len--) buffer[ptrOut++] = z;	else break;
	}

	return 	(unsigned char*)&buffer[3];
}


/* --- uart_putchar -------------------------------------------------------- */

/* Write char on the serial port #1 (used by printf for debug) */
int
uart_putchar(char c, FILE *stream)
{
  if (c == '\n') uart_putchar('\r', stream);

  loop_until_bit_is_set(UCSR1A, UDRE1);
  UDR1 = c;
  return 0;
}


void UART_Init (void)
//############################################################################
{

	//Enable UART0 for transmission and reception
	UCSR0B = (1 << TXEN0) | (1 << RXEN0);
	//with corresponding interrupts
	UCSR0B |= (1<<RXCIE0) | (1<<TXCIE0);
	//UART0 Double Speed (U2X)
	UCSR0A |= (1<<U2X0);

	//Set baudrate for UART0
	UBRR0H= (uint8_t)(ubrr >> 8);
	UBRR0L= (uint8_t)ubrr;

	//Enable UART1 for transmission only
	UCSR1B =(1 << TXEN1);
	//with corresponding interrupt
	UCSR1B |= (1<<TXCIE1);
	//UART1 Double Speed (U2X)
	UCSR1A |= (1<<U2X1);

	//Set baudrate for UART1
	UBRR1H= (uint8_t)(ubrr >> 8);
	UBRR1L= (uint8_t)ubrr;

}


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++ IMU ISR
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
SIGNAL(USART1_TX_vect)
{
	static unsigned int ptr = 0;
	unsigned char tmp_tx;
	if(!TransmissionComplete1)
	{
		ptr++;
		tmp_tx = TxdBuffer1[ptr];
		if((tmp_tx == '\r') || (ptr == MAX_TX_BUFF))
		{
			ptr = 0;
			TransmissionComplete1 = 1;
		}
		UDR1 = tmp_tx;
	}
	else ptr = 0;
}


void IMUTransmission(void)
{

	if(TransmissionComplete1) // && FlightModeOptions.dataTime!=0
	{
		imuTransPerMsDecs = 0;
		sendPackets1++;


		// Fill field. Make sure that \r is not present!


	//NEW CODE (Paolo S.)
// 		signed int* mikKopDataPtr = (signed int*) &mikKopData;
// 		signed int mirrorData[MIK_KOP_DATA_PACKET_SIZE];


// 		for(int i=0; i<MIK_KOP_DATA_PACKET_SIZE ; i++ ){
// 			if(activeDataIds[i] < mikKopDataSize){
// 				mirrorData[i] = (signed int) mikKopDataPtr[activeDataIds[i]];
// 			}
// 		}
//
// 		// Gyro data
// 		TxdBuffer1[0] = mirrorData[0] >> 4;
// 		TxdBuffer1[1] = ((mirrorData[0] & 0x0f) << 2) + (mirrorData[1] >> 8);
// 		TxdBuffer1[2] = (mirrorData[1] >> 2) & 0x3f;
// 		TxdBuffer1[3] = ((mirrorData[1] & 0x03) << 4) + (mirrorData[2] >> 6);
// 		TxdBuffer1[4] = mirrorData[2] & 0x3f;
//
// 		// Accelerometer data
// 		TxdBuffer1[5] = mirrorData[3] >> 4;
// 		TxdBuffer1[6] = ((mirrorData[3] & 0x0f) << 2) + (mirrorData[4] >> 8);
// 		TxdBuffer1[7] = (mirrorData[4] >> 2) & 0x3f;
// 		TxdBuffer1[8] = ((mirrorData[4] & 0x03) << 4) + (mirrorData[5] >> 6);
// 		TxdBuffer1[9] = mirrorData[5] & 0x3f;

	//OLD CODE (Riccardo)
		// Gyro data
		TxdBuffer1[0] = mikKopData.rawGyroX >> 4;
		TxdBuffer1[1] = ((mikKopData.rawGyroX & 0x0f) << 2) + (mikKopData.rawGyroY >> 8);
		TxdBuffer1[2] = (mikKopData.rawGyroY >> 2) & 0x3f;
		TxdBuffer1[3] = ((mikKopData.rawGyroY & 0x03) << 4) + (mikKopData.rawGyroZ >> 6);
		TxdBuffer1[4] = mikKopData.rawGyroZ & 0x3f;

		// Accelerometer data
		TxdBuffer1[5] = mikKopData.rawAccX >> 4;
		TxdBuffer1[6] = ((mikKopData.rawAccX & 0x0f) << 2) + (mikKopData.rawAccY >> 8);
		TxdBuffer1[7] = (mikKopData.rawAccY >> 2) & 0x3f;
		TxdBuffer1[8] = ((mikKopData.rawAccY & 0x03) << 4) + (mikKopData.rawAccZ >> 6);
		TxdBuffer1[9] = mikKopData.rawAccZ & 0x3f;

		// Battery voltage (now managed by mkinterface)
		//TxdBuffer1[10] = (unsigned char)(mikKopData.battVoltage-106) & 0x3f;
                int i;
		for (i = 0; i < 10; i++) {
			TxdBuffer1[i] += '=';
		}

		// Add CRC
		const unsigned int tempCRC = computeCRC(TxdBuffer1,10);
		TxdBuffer1[10]= '=' + (tempCRC & 0x3F); // only lower 6 bit
		TxdBuffer1[11] = '\r';

		//Start transmitting
		TransmissionComplete1 = 0;
		UDR1 = TxdBuffer1[0];

	}
}




//----------------------------------


SIGNAL(USART0_TX_vect)
{
	static unsigned int ptr = 0;
	unsigned char tmp_tx;
	if(!TransmissionComplete0)
	{
		ptr++;
		tmp_tx = TxdBuffer0[ptr];
		if((tmp_tx == '\r') || (ptr == MAX_TX_BUFF))
		{
			ptr = 0;
			TransmissionComplete0 = 1;
		}
		UDR0 = tmp_tx;
	}
	else ptr = 0;
}


// ----- Interrupt service for the main serial -------------//

SIGNAL(USART0_RX_vect)
{

	static unsigned char UartState = 0, buf_ptr;

	const unsigned char SioTmp = UDR0;

	if(buf_ptr >= MAX_TX_BUFF){
		UartState = 0;
	}


	if(SioTmp == '\r' && UartState == 2){ // Received terminating char
		UartState = 0;
		if (checkCRC(RxdBuffer0,buf_ptr)){
			if(!newDataReceived0){
				//printf("New data received\r\n");
				newDataReceived0 = 1;
				numReceivedBytes0 = buf_ptr + 1;
				RxdBuffer0[buf_ptr] = '\r';
			}
		} else {
			//printf("Wrong CRC\r\n");
			mikKopData.cntCRCErrors++;
		};
	}else{

		switch(UartState)
		{
			case 0: // Waiting for new message
				if(SioTmp == '#' && !newDataReceived0){  // Start char received and old data already processed
					UartState++;
				}
				buf_ptr = 0;
				RxdBuffer0[buf_ptr++] = SioTmp;
				break;
			case 1: // Evaluating address
				UartState++;
				RxdBuffer0[buf_ptr++] = SioTmp;
				break;
			case 2: // Receiving data
				RxdBuffer0[buf_ptr] = SioTmp;
				if(buf_ptr < MAX_RX_BUFF){
					buf_ptr++;
				} else {
					UartState = 0;
				}
				break;
			default:
				UartState = 0;
				break;
		}

	}
}


void processRxData(void)
{
	if(!newDataReceived0){
		return;
	}

	//printf("Processing command ");

	extCmdProcessPerMsDecs = 0;

	unsigned char* pRxData=0;

	// decode data in the input buffer
	pRxData = decode64(RxdBuffer0,numReceivedBytes0);

	// Reset emergency timer and update statistics
	if (RxdBuffer0[2] == 'b' || RxdBuffer0[2] == 'l' || RxdBuffer0[2] == 'u' || RxdBuffer0[2] == 's') {
		if (CountMillisecondsLastRx0 > mikKopData.maxReceiveCmdPeriodMs){
			mikKopData.maxReceiveCmdPeriodMs = CountMillisecondsLastRx0;
		}
		CountMillisecondsLastRx0 = 0;
		receivedCmds++;
	}


	// TODO: RxdBuffer0[1] is ignored, could be used for something else

	switch(RxdBuffer0[2])
	{

		case 'b': // update commands
			memcpy((unsigned char *)&cmdBuf, (unsigned char *)pRxData, sizeof(cmdBuf));

			//Debug print
			//printf("'b': update commands",cmdBuf.cmd0);
			//printf("cmdBuf: %03d %03d %03d %03d \r\n",cmdBuf.cmd0,cmdBuf.cmd1,cmdBuf.cmd2,cmdBuf.cmd3);

			break;

		case 'l': // Set active data ids
			memcpy((unsigned char *)&commandsPlusDataIds, (unsigned char *)pRxData, sizeof(commandsPlusDataIds));
			// update commands
			memcpy((unsigned char *)&cmdBuf, (unsigned char *)&commandsPlusDataIds, sizeof(cmdBuf));
			// update activeDataIds
			memcpy((unsigned char *)&activeDataIds, (unsigned char *)commandsPlusDataIds.activeDataIds, sizeof(activeDataIds));

			//TODO check for valid IDs

			confirmActiveDataIds = 1;

			//Debug print
			//printf("'l': set activeDataIds to ");
			//for (short unsigned int j=0; j<sizeof(commandsPlusDataIds.activeDataIds); printf("%s ",mikKopDataNames(commandsPlusDataIds.activeDataIds[j])), j++);
			//printf("\r\n");

			break;

		case 'u': // Send back a single value with the requested id
		{
			memcpy((unsigned char *)&commandsPlusId, (unsigned char *)pRxData, sizeof(commandsPlusId));
			// update commands
			memcpy((unsigned char *)&cmdBuf, (unsigned char *)&commandsPlusId, sizeof(cmdBuf));

			singleUpdateValue.id = commandsPlusId.id;

			if (singleUpdateValue.id < mikKopDataSize) {
				confirmSingleUpdateValue=1;
			}

			//Debug print
			//printf("'u': send back a single value with id %s\r\n",mikKopDataNames(commandsPlusId.id));

			break;
		}

		case 's': // Set a single value and send it back
		{
			memcpy((unsigned char *)&commandsPlusValue, (unsigned char *)pRxData, sizeof(commandsPlusValue));
			// update commands
			memcpy((unsigned char *)&cmdBuf, (unsigned char *)&commandsPlusValue, sizeof(cmdBuf));

			singleSetValue.id = commandsPlusValue.id;
			singleSetValue.value = commandsPlusValue.value;

			if(singleSetValue.id < mikKopDataSize){
				setValueByID(singleSetValue.id, singleSetValue.value);
				confirmSingleSetValue=1;
			}

			//Debug print
			//printf("'s': set a single value and send it back with id %s and value %d\r\n", mikKopDataNames(commandsPlusValue.id), commandsPlusValue.value);

			break;
		}

		default:
			//printf("'%c': unknown command", RxdBuffer0[2]);
			break;

	}

	newDataReceived0 = 0;
}


void sendOutData(unsigned char cmd, unsigned char address, unsigned char bufferNumber, ...) //unsigned char *snd, unsigned char len)
{

	// Update statistics
	sendPackets++;
	if (CountMillisecondsLastTx0 > mikKopData.maxSendPeriodMs){
		mikKopData.maxSendPeriodMs = CountMillisecondsLastTx0;
	}
	CountMillisecondsLastTx0 = 0;

	va_list ap;
	unsigned int pt = 0; // change from 1. I didn't see any reason.
	unsigned char a,b,c;
	unsigned char ptr = 0;

	unsigned char *snd = 0;
	int len = 0;

	TxdBuffer0[pt++] = '#';				// start char
	TxdBuffer0[pt++] = 'a' + address;	// address
	TxdBuffer0[pt++] = cmd;				// command

	va_start(ap, bufferNumber);
	if(bufferNumber)
	{
		snd = va_arg(ap, unsigned char*);
		len = va_arg(ap, int);
		ptr = 0;
		bufferNumber--;
	}
	while(len)
	{
		if(len)
		{
			a = snd[ptr++];
			len--;
			if((!len) && bufferNumber)
			{
				snd = va_arg(ap, unsigned char*);
				len = va_arg(ap, int);
				ptr = 0;
				bufferNumber--;
			}
		}
		else a = 0;
		if(len)
		{
			b = snd[ptr++];
			len--;
			if((!len) && bufferNumber)
			{
				snd = va_arg(ap, unsigned char*);
				len = va_arg(ap, int);
				ptr = 0;
				bufferNumber--;
			}
		}
		else b = 0;
		if(len)
		{
			c = snd[ptr++];
			len--;
			if((!len) && bufferNumber)
			{
				snd = va_arg(ap, unsigned char*);
				len = va_arg(ap, int);
				ptr = 0;
				bufferNumber--;
			}
		}
		else c = 0;
		TxdBuffer0[pt++] = '=' + (a >> 2);
		TxdBuffer0[pt++] = '=' + (((a & 0x03) << 4) | ((b & 0xf0) >> 4));
		TxdBuffer0[pt++] = '=' + (((b & 0x0f) << 2) | ((c & 0xc0) >> 6));
		TxdBuffer0[pt++] = '=' + ( c & 0x3f);
	}

	va_end(ap);

	// Add CRC and terminating character at the end of the message
	const unsigned int tmpCRC = computeCRC(TxdBuffer0,pt);
	TxdBuffer0[pt++] = '=' + (tmpCRC >> 6);
	TxdBuffer0[pt++] = '=' + (tmpCRC & 0x3F);
	TxdBuffer0[pt++] = '\r';

	// Start transmission
	TransmissionComplete0 = 0;
	UDR0 = TxdBuffer0[0];
}



void sendTxData0(void)
{

	if(!TransmissionComplete0) return;

	sendDataPerMsDecs = 0;

	if(mikKopData.mirrorDataActive && (mirrorTimer > mikKopData.dataTimePeriod) && TransmissionComplete0) // && FlightModeOptions.dataTime!=0
	{


		signed int* mikKopDataPtr = (signed int*) &mikKopData;
		signed int mirrorData[MIK_KOP_DATA_PACKET_SIZE];

                int i;
		for(i=0; i<MIK_KOP_DATA_PACKET_SIZE ; i++ ){
			if(activeDataIds[i] < mikKopDataSize){
				mirrorData[i] = (signed int) mikKopDataPtr[activeDataIds[i]];
			}
		}

		sendOutData('D', FC_ADDRESS, 1, (unsigned char *) &mirrorData,sizeof(mirrorData));
/*
		printf("Sending mirror information:\r\n");
		for(i=0; i<MIK_KOP_DATA_PACKET_SIZE ; i++ ){
			if(activeDataIds[i] < mikKopDataSize){
				printf("id: %s, value: %d\r\n",mikKopDataNames(activeDataIds[i]),getValueByID(activeDataIds[i]));
			}
		}
*/
		mirrorTimer = 0;

	}

	if(confirmActiveDataIds && TransmissionComplete0)
	{
		/*printf("Sending back activeDataIds confirmation ");
		for (short unsigned int j=0; j<sizeof(commandsPlusDataIds.activeDataIds); printf("%s ",mikKopDataNames(commandsPlusDataIds.activeDataIds[j])), j++);
		printf("\r\n");*/
		sendOutData('L', FC_ADDRESS, 1, (unsigned char *) &activeDataIds,sizeof(activeDataIds));
		confirmActiveDataIds=0;

	}

	if(confirmSingleUpdateValue && TransmissionComplete0)
	{

		//signed int* mikKopDataPtr = (signed int*) &mikKopData;
		//singleUpdateValue.value = (signed int) *(mikKopDataPtr + singleUpdateValue.id);
		singleUpdateValue.value = getValueByID(singleUpdateValue.id);

		//printf("Sending back requested single value with id %s and value %d\r\n", mikKopDataNames(singleUpdateValue.id), singleUpdateValue.value);
		sendOutData('S', FC_ADDRESS, 1, (unsigned char *) &singleUpdateValue,sizeof(singleUpdateValue));
		confirmSingleUpdateValue=0;

	}

	if(confirmSingleSetValue && TransmissionComplete0)
	{

		//signed int* mikKopDataPtr = (signed int*) &mikKopData;
		//singleSetValue.value = (signed int) *(mikKopDataPtr + singleSetValue.id);
		singleSetValue.value = getValueByID(singleSetValue.id);

		//printf("Sending back confirmation of set single value with id %s and value %d\r\n", mikKopDataNames(singleSetValue.id), singleSetValue.value);
		sendOutData('S', FC_ADDRESS, 1, (unsigned char *) &singleSetValue,sizeof(singleSetValue));
		confirmSingleSetValue=0;

	}
}
